function [alphac,alphaw,alphao,alphaz,alphan,fg]=Discretiseer(Model,nx,ny)
%[alphac,alphaw,alphao,alphaz,alphan,fg]=Discretiseer(Model,nx,ny)
%
% De functie "Model" levert de volgende grootheden
%
% X0 en Y0 voor het gebied D=[0,X0]x[0,Y0]
% diffusie coefficienten a,b
% advectie veld (u,v)
% absorbtie constante c
% bron term f (regenbui), 
%
% pomp=[x1,y1,q1;x2,y2,q2;...]; 
% rivier=[alpha1,beta1,gamma1,q1;alpha2,beta2,gamma2,q2;...];
%
% en randvoorwaarden
%
%
% -d_x(a d_x phi) - d_y(b d_y phi)
%      + d_x(u phi) + d_y(v phi) + c phi = f
%                       op D=[0,X0]x[0,Y0]
%
% Essentiel randwaarden voor x=0,x=X0:
%    phi=phiw voor x=0;         phi=phio voor x=X0;
% Natuurlijke randvoorwaarden voor y=0 en y=Y0:
%   -(b d_y phi)=phiz voor y=0; (b d_y phi)=phin voor y=Y0;
%
% nx aantal punten in het inwendig rooster in de x richting (west-oost)
% ny aantal punten in het inwendig rooster in de y richting  (zuid noord)
%___________________________________________________________________
%
%
% Bereken in het (i,j)-de rooster punt (i*hx,j*hy) de stencils
%
%   |   0    alphan     0    |
%   | alphaw alphac   alphao |   phi = fg
%   |   0    alphaz     0    |
%
% hierbij is hx=X0/(nx+1), hy=Y0/(ny+1) en i=1:nx; j=1:ny;
%
%

%
% Gerard Sleijpen, september 2010
%


%% Vind de grootte van het gebied
[X0,Y0]=feval(Model,'gebied');

%% Bepaal de maaswijdtes (of stapgroottes) hx en hy
hx=X0/(nx+1); hy=Y0/(ny+1);


i=1:nx;       j=1:ny;
%% Maak de coordinaten voor de roosterlijnen
%% xi(i)=i*hx, yj(j)=j*hy;
xi=i*hx;      yj=j*hy;

%% Bepaal de roosterpunten uit de roosterlijnen
%% x(i,j)=xi(i)=i*hx, y(i,j)=yj(j)=j*hy
[x,y]=ndgrid(xi,yj);


%% We berekenen eerste de a, b, u, v, c en f termen
%% in geschikte roosterpunten. De resulterende
%% grootheden zijn gemarkeerd met een g (van grid):
%% Zo is fg de functie f uitgerekend in geschikte roosterpunten

   %% de absorbtie term en de bronterm
%% Onderstaand twe regels berekenen cg en fg zodat
%%   cg(i,j)=c(x(i,j),y(i,j))=c(i*hx,j*hy) 
%%   fg(i,j)=f(x(i,j),y(i,j))=f(i*hx,j*hy)
cg=feval(Model,x,y,'absorbtie');
fg=feval(Model,x,y,'bronterm');

   %% de advectie termen
%% Onderstaand stukje code berekend ug, vg, de u en v snelheids
%% vector componenten op het rooster.
%% Er geldt
%%      ug(i,j)=u(i*hx-hx,j*hy)
%%      vg(i,j)=v(i*hx,j*hy-hy)
[U,V,vek]=feval(Model,[],[],'advectie');
if vek==1 
   %% Als het stromingsveld is uitgerekend (uit de grondwatervergelijkingen)
   if vekk==1
      %% U(i,j)=u((i-0.5)*hx,j*hy)
      %% middel om de u waarde te vinden in (i*hx,j*hy)
      %% ug(i,j)=u(i*hx,j*hy)
    ug(i+1,j)=(U(i+1,j)+U(i,j))/2;
      %% extrapoleer naar de de west- en oostrand
    ug(1,j)=2*ug(2,j)-ug(3,j); ug(nx+2,j)=2*ug(nx+1,j)-ug(nx,j);
      %% V(i,j)=v(i*hx,(j-0.5)*hy)
      %% middel om de v waarde te vinden in (i*hx,j*hy)
      %% vg(i,j)=vu(i*hx,j*hy)
    vg(i,j+1)=(V(i,j+1)+V(i,j))/2;
      %% extrapoleer naar de de noord- en zuidrand
    vg(i,1)=2*vg(i,2)-vg(i,3); vg(i,ny+2)=2*vg(i,ny+1)-vg(i,ny);
    vekk=0;
   end
else
   %% als het stromingsveld gedefinieerd is als een functie
   [x,y]=ndgrid([0:hx:X0],yj); 
   [ug,vg0]=feval(Model,x,y,'advectie'); 
   [x,y]=ndgrid(xi,[0:hy:Y0]); 
   [ug0,vg]=feval(Model,x,y,'advectie'); 
   vekk=1;
end
%% einde advectie 

  %% bereken de doorlaadbaarheidscoefficient a 
  %% in "0.5" roosterpunten is x-richting:
  %% ag(i,j)=a((-0.5+i)*hx,j*hy)
[xh,y]=ndgrid([0.5*hx:hx:X0],yj); 
[ag,bg0]=feval(Model,xh,y,'diffusie');

  %% bereken de doorlaadbaarheidscoefficient b 
  %% in "0.5" roosterpunten in de y-richting:
  %% bg(i,j)=b(j*hx,(-0.5+j)*hy)
[x,yh]=ndgrid(xi,[0.5*hy:hy:Y0]); 
[ag0,bg]=feval(Model,x,yh,'diffusie');


  %%% Bepaal de STECIL coeeficienten van de PDE
  %%%
%% inmiddels zijn beschikbaar arrays ag,bg,ug,vg,cg,fg waarvoor
%%   ag(i,j)=a((-0.5+i)*hx,j*hy)              (i=1:nx+1,  j=1:ny)
%%   bg(i,j)=b(j*hx,(-0.5+j)*hy)              (i=1:nx,    j=1:ny+1)
%%   ug(i,j)=u(i*hx-hx,j*hy)                  (i=1:nx+2,  j=1:ny)
%%   vg(i,j)=v(i*hx,j*hy-hy)                  (i=1:nx,    j=1:ny+2)
%%   cg(i,j)=c(x(i,j),y(i,j))=c(i*hx,j*hy)    (i=1:nx,    j=1:ny)
%%   fg(i,j)=f(x(i,j),y(i,j))=f(i*hx,j*hy)    (i=1:nx,    j=1:ny)
%% Let op: sommige roosterpunten zijn "verschoven": In MATLAB mag
%% een arrayindex niet met index 0 beginnen.


alphaw=zeros(nx,ny);
alphac=zeros(nx,ny);
alphao=zeros(nx,ny);
alphaz=zeros(nx,ny);
alphan=zeros(nx,ny);
for j=1:ny
  for i=1:nx
     %% uit de advectie
     alphaw(i,j)=-ug(i,j)/(2*hx);

 %%%%%%%%%%% ??????????

     %% uit de diffusie

 %%%%%%%%%%% ??????????

  end
end


  %% Gemengde randwaarden
  %
  %% We berekenen functies als psiw en muw die gedefinieerd
  %% zijn op de west rand in roosterpunten op deze rand. 
  %% De roosterfunctie versie is genoteerd als psiwg en muwg.
  %% Er geldt, voor j=1:ny
  %%    psiwg(j)=psiw(j*hy)
  %%    muwg(j)=muw(j*hy)
  %% Evenzo voor de andere randen
  [psiwg,muwg]=feval(Model,yj,'westrand'); 
  [psiog,muog]=feval(Model,yj,'oostrand'); 
  [psing,mung]=feval(Model,xi','noordrand'); 
  [psizg,muzg]=feval(Model,xi','zuidrand'); 
  %% gebruik de randvoorwaarden om de roosterfuncties psi
  %% op de rand te elimineren


for j=1:ny
  %% west rand 
  teller=muwg(j)*ag(1,j)/hx; noemer=1-muwg(j)-teller; 
  fg(1,j)=fg(1,j)- %%%%%% ???????
  alphac(1,j)=alphac(1,j)- %%%%% ???????
  alphaw(1,j)=0*alphaw(1,j);

  %% oost rand

 %%%%%%%%%%% ??????????

end

%% zuid rand

%%%%%%%%%%% ??????????

%% noord rand
 
%%%%%%%%%%% ??????????



if 1
  %% controleer de waarden van een aantal stencils
  stencil(alphac,alphaw,alphao,alphaz,alphan,nx,ny);
end

 %%
 %% Brontermen
 %%
 %% Bouw de pompen in
pomp=feval(Model,'pomp');
if ~isempty(pomp)
  [k,l]=size(pomp);
  if l==3
    for kk=1:k
 
%%%%%%%%%%% ??????????

    end
  end
end

  %% bouw de rivier in
rivier=feval(Model,'rivier');
if ~isempty(rivier)
  [k1,l1]=size(rivier);
  for kk=1:k1
    [S,IndexRoosterpunt,Lengte]=Rivier(rivier(kk,:),nx,ny,X0,Y0);
    kub=rivier(kk,4)/(hx*hy); 

    l=size(IndexRoosterpunt,2);
    for k=1:l-1
      i=IndexRoosterpunt(1,k); j=IndexRoosterpunt(2,k);
      fg(i,j)=fg(i,j)+Lengte(k)*kub;
    end
    
  end
end

return
%==================================================
function stencil(alphac,alphaw,alphao,alphaz,alphan,i1,j1)
% toon de stecils roind (i1,j1)

  [nx,ny]=size(alphac);

  for i=max(1,i1-2):min(i1,nx)
    for j=max(1,ny-2):min(j1,ny)
      fprintf('\n\nStencil(%d,%d):',i,j)
      fprintf('\n  %10s %10.2e %10s','',alphan(i,j),'')
      fprintf('\n  %10.2e %10.2e %10.2e',alphaw(i,j),alphac(i,j),alphao(i,j))
      fprintf('\n  %10s %10.2e %10s','',alphaz(i,j),'')
    end
  end
  fprintf('\n\n')

return
%====================================================

